#!/usr/bin/env python
import pytest
from netmiko import file_transfer

# def test_enable_scp(scp_fixture):
#    ssh_conn, scp_transfer = scp_fixture
#
#    scp_transfer.disable_scp()
#    output = ssh_conn.send_command_expect("show run | inc scp")
#    assert 'ip scp server enable' not in output
#
#    scp_transfer.enable_scp()
#    output = ssh_conn.send_command_expect("show run | inc scp")
#    assert 'ip scp server enable' in output


def test_scp_put(scp_fixture):
    ssh_conn, scp_transfer = scp_fixture
    if scp_transfer.check_file_exists():
        assert False
    else:
        scp_transfer.put_file()
        assert scp_transfer.check_file_exists() is True


def test_remote_space_available(scp_fixture, expected_responses):
    ssh_conn, scp_transfer = scp_fixture
    remote_space = scp_transfer.remote_space_available()
    assert remote_space >= expected_responses["scp_remote_space"]


def test_local_space_available(scp_fixture):
    ssh_conn, scp_transfer = scp_fixture
    local_space = scp_transfer.local_space_available()
    assert local_space >= 1000000000


def test_verify_space_available_put(scp_fixture):
    ssh_conn, scp_transfer = scp_fixture
    assert scp_transfer.verify_space_available() is True
    # intentional make there not be enough space available
    scp_transfer.file_size = 100000000000
    assert scp_transfer.verify_space_available() is False


def test_remote_file_size(scp_fixture):
    ssh_conn, scp_transfer = scp_fixture
    if not scp_transfer.check_file_exists():
        scp_transfer.put_file()
    remote_file_size = scp_transfer.remote_file_size()
    assert remote_file_size == 19


def test_md5_methods(scp_fixture):
    ssh_conn, scp_transfer = scp_fixture
    md5_value = "d8df36973ff832b564ad84642d07a261"

    remote_md5 = scp_transfer.remote_md5()
    assert remote_md5 == md5_value
    assert scp_transfer.compare_md5() is True


def test_disconnect(scp_fixture):
    """Terminate the SSH session."""
    ssh_conn, scp_transfer = scp_fixture
    ssh_conn.disconnect()


def test_verify_space_available_get(scp_fixture_get):
    ssh_conn, scp_transfer = scp_fixture_get
    assert scp_transfer.verify_space_available() is True
    # intentional make there not be enough space available
    scp_transfer.file_size = 100000000000000
    assert scp_transfer.verify_space_available() is False


def test_scp_get(scp_fixture_get):
    ssh_conn, scp_transfer = scp_fixture_get

    if scp_transfer.check_file_exists():
        # File should not already exist
        assert False
    else:
        scp_transfer.get_file()
        if scp_transfer.check_file_exists():
            assert True
        else:
            assert False


def test_md5_methods_get(scp_fixture_get):
    ssh_conn, scp_transfer = scp_fixture_get
    md5_value = "d8df36973ff832b564ad84642d07a261"
    local_md5 = scp_transfer.file_md5("test9.txt")
    assert local_md5 == md5_value
    assert scp_transfer.compare_md5() is True


def test_disconnect_get(scp_fixture_get):
    """Terminate the SSH session."""
    ssh_conn, scp_transfer = scp_fixture_get
    ssh_conn.disconnect()


def test_file_transfer(scp_file_transfer):
    """Test Netmiko file_transfer function."""
    ssh_conn, file_system = scp_file_transfer
    source_file = "test9.txt"
    dest_file = "test9.txt"
    direction = "put"

    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )

    # No file on device at the beginning
    assert (
        transfer_dict["file_exists"]
        and transfer_dict["file_transferred"]
        and transfer_dict["file_verified"]
    )

    # File exists on device at this point
    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )
    assert (
        transfer_dict["file_exists"]
        and not transfer_dict["file_transferred"]
        and transfer_dict["file_verified"]
    )

    # Don't allow a file overwrite (switch the source file, but same dest file name)
    source_file = "test2_src.txt"
    with pytest.raises(Exception):
        transfer_dict = file_transfer(
            ssh_conn,
            source_file=source_file,
            dest_file=dest_file,
            file_system=file_system,
            direction=direction,
            overwrite_file=False,
        )

    # Don't allow MD5 and file overwrite not allowed
    source_file = "test9.txt"
    with pytest.raises(Exception):
        transfer_dict = file_transfer(
            ssh_conn,
            source_file=source_file,
            dest_file=dest_file,
            disable_md5=True,
            file_system=file_system,
            direction=direction,
            overwrite_file=False,
        )

    # Don't allow MD5 (this will force a re-transfer)
    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        disable_md5=True,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )
    assert (
        transfer_dict["file_exists"]
        and transfer_dict["file_transferred"]
        and not transfer_dict["file_verified"]
    )

    # Transfer 'test2.txt' in preparation for get operations
    source_file = "test2_src.txt"
    dest_file = "test2.txt"
    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )
    assert transfer_dict["file_exists"]

    # GET Operations
    direction = "get"
    source_file = "test9.txt"
    dest_file = "testx.txt"
    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        disable_md5=False,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )
    # File get should occur here
    assert (
        transfer_dict["file_exists"]
        and transfer_dict["file_transferred"]
        and transfer_dict["file_verified"]
    )

    # File should exist now
    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        disable_md5=False,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )
    assert (
        transfer_dict["file_exists"]
        and not transfer_dict["file_transferred"]
        and transfer_dict["file_verified"]
    )

    # Don't allow a file overwrite (switch the file, but same dest file name)
    source_file = "test2.txt"
    dest_file = "testx.txt"
    with pytest.raises(Exception):
        transfer_dict = file_transfer(
            ssh_conn,
            source_file=source_file,
            dest_file=dest_file,
            file_system=file_system,
            direction=direction,
            overwrite_file=False,
        )

    # Don't allow MD5 and file overwrite not allowed
    source_file = "test9.txt"
    dest_file = "testx.txt"
    with pytest.raises(Exception):
        transfer_dict = file_transfer(
            ssh_conn,
            source_file=source_file,
            dest_file=dest_file,
            disable_md5=True,
            file_system=file_system,
            direction=direction,
            overwrite_file=False,
        )

    # Don't allow MD5 (this will force a re-transfer)
    transfer_dict = file_transfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        disable_md5=True,
        file_system=file_system,
        direction=direction,
        overwrite_file=True,
    )
    assert (
        transfer_dict["file_exists"]
        and transfer_dict["file_transferred"]
        and not transfer_dict["file_verified"]
    )



#!/usr/bin/env python

import os
from os.path import dirname, join, relpath

from netmiko import utilities
from netmiko._textfsm import _clitable as clitable

RESOURCE_FOLDER = join(dirname(dirname(__file__)), "etc")
RELATIVE_RESOURCE_FOLDER = join(dirname(dirname(relpath(__file__))), "etc")
CONFIG_FILENAME = join(RESOURCE_FOLDER, ".netmiko.yml")


def test_load_yaml_file():
    """Read a YAML file successfully"""
    filename = join(RESOURCE_FOLDER, "yaml_test.yml")
    expected = {
        "answer": 42,
        "hello": "world",
        "complex": {"truth": False, "key": "value"},
    }
    assert utilities.load_yaml_file(filename) == expected


def test_invalid_yaml_file():
    """Try to read an invalid YAML file"""
    filename = join(RESOURCE_FOLDER, "this_should_not_exist.yml")
    try:
        utilities.load_yaml_file(filename)
    except SystemExit as exc:
        assert isinstance(exc, SystemExit)
        return
    assert False


def test_find_cfg_file():
    """
    Search for netmiko_tools config file in the following order:

    NETMIKO_TOOLS_CFG environment variable
    Current directory
    Home directory

    Look for file named: .netmiko.yml or netmiko.yml
    """
    # Search using environment variable (point directly at end file)
    os.environ["NETMIKO_TOOLS_CFG"] = join(RESOURCE_FOLDER, ".netmiko.yml")
    assert utilities.find_cfg_file() == CONFIG_FILENAME

    # Search using environment variable (pointing at directory)
    os.environ["NETMIKO_TOOLS_CFG"] = RESOURCE_FOLDER
    assert utilities.find_cfg_file() == CONFIG_FILENAME

    try:
        cwd = os.getcwd()
        os.chdir(dirname(__file__))

        # Environment var should be preferred over current dir
        assert utilities.find_cfg_file() == CONFIG_FILENAME

        #  Delete env var and verify current dir is returned
        del os.environ["NETMIKO_TOOLS_CFG"]
        assert utilities.find_cfg_file() == "./.netmiko.yml"
    finally:
        # Change directory back to previous state
        os.chdir(cwd)

    # Verify explicit call using full filename
    assert utilities.find_cfg_file(CONFIG_FILENAME) == CONFIG_FILENAME


def test_load_cfg_file():
    """Try to load a configuration file"""
    expected = {
        "rtr1": {
            "device_type": "cisco_ios",
            "ip": "10.10.10.1",
            "username": "admin",
            "password": "cisco123",
            "secret": "cisco123",
        },
        "rtr2": {
            "device_type": "cisco_ios",
            "ip": "10.10.10.2",
            "username": "admin",
            "password": "cisco123",
            "secret": "cisco123",
        },
        "cisco": ["rtr1", "rtr2"],
    }
    assert utilities.load_devices(CONFIG_FILENAME) == expected


def test_obtain_all_devices():
    """Dynamically create 'all' group."""
    netmiko_tools_load = utilities.load_devices(CONFIG_FILENAME)
    expected = {
        "rtr1": {
            "device_type": "cisco_ios",
            "ip": "10.10.10.1",
            "username": "admin",
            "password": "cisco123",
            "secret": "cisco123",
        },
        "rtr2": {
            "device_type": "cisco_ios",
            "ip": "10.10.10.2",
            "username": "admin",
            "password": "cisco123",
            "secret": "cisco123",
        },
    }
    result = utilities.obtain_all_devices(netmiko_tools_load)
    assert result == expected


def test_find_netmiko_dir():
    """Try to get the netmiko_dir"""
    folder = dirname(__file__)
    os.environ["NETMIKO_DIR"] = folder
    result = utilities.find_netmiko_dir()
    assert result[0] == folder
    assert result[1].endswith("/tmp")


def test_invalid_netmiko_dir():
    """Try with an invalid netmiko_base_dir"""
    os.environ["NETMIKO_DIR"] = "/"
    try:
        utilities.find_netmiko_dir()
    except ValueError as exc:
        assert isinstance(exc, ValueError)
        return
    assert False


def test_string_to_bytes():
    """Convert string to bytes"""
    assert utilities.write_bytes("test") == b"test"


def test_bytes_to_bytes():
    """Convert bytes to bytes"""
    result = b"hello world"
    assert utilities.write_bytes(result) == result


def test_invalid_data_to_bytes():
    """Convert an invalid data type to bytes"""
    try:
        utilities.write_bytes(456_779)
    except ValueError as exc:
        assert isinstance(exc, ValueError)
        return

    assert False


def test_ensure_resource_dir_exists():
    """Ensure that the resource folder exists"""
    utilities.ensure_dir_exists(RESOURCE_FOLDER)


def test_ensure_file_exists():
    """Ensure that a file makes ensure_dir_exists raise an error"""
    try:
        utilities.ensure_dir_exists(__file__)
    except ValueError as exc:
        assert isinstance(exc, ValueError)
        return
    assert False


def test_clitable_to_dict():
    """Converts TextFSM cli_table object to list of dictionaries"""
    table = clitable.CliTable(template_dir=RESOURCE_FOLDER)
    text_filename = join(RESOURCE_FOLDER, "textfsm.txt")
    template_filename = join(RESOURCE_FOLDER, "cisco_ios_show_version.template")
    with open(text_filename) as data_file:
        text = data_file.read()

    with open(template_filename) as template_file:
        table = table._ParseCmdItem(text, template_file)

    result = utilities.clitable_to_dict(table)
    assert result == [{"model": "4500"}]


def test_textfsm_w_index():
    """Convert raw CLI output to structured data using TextFSM template"""
    os.environ["NET_TEXTFSM"] = RESOURCE_FOLDER
    raw_output = "Cisco IOS Software, Catalyst 4500 L3 Switch Software"
    result = utilities.get_structured_data(
        raw_output, platform="cisco_ios", command="show version"
    )
    assert result == [{"model": "4500"}]


def test_textfsm_index_relative_path():
    """Test relative path for textfsm ntc directory"""
    os.environ["NET_TEXTFSM"] = RELATIVE_RESOURCE_FOLDER
    raw_output = "Cisco IOS Software, Catalyst 4500 L3 Switch Software"
    result = utilities.get_structured_data(
        raw_output, platform="cisco_ios", command="show version"
    )
    assert result == [{"model": "4500"}]


def test_textfsm_direct_template():
    """Convert raw CLI output to structured data using TextFSM template (no index)."""
    raw_output = "Cisco IOS Software, Catalyst 4500 L3 Switch Software"
    result = utilities.get_structured_data(
        raw_output,
        platform="cisco_ios",
        command="show version",
        template=f"{RESOURCE_FOLDER}/cisco_ios_show_version.template",
    )
    assert result == [{"model": "4500"}]

    # Should also work with no-platform or command
    result = utilities.get_structured_data(
        raw_output, template=f"{RESOURCE_FOLDER}/cisco_ios_show_version.template"
    )
    assert result == [{"model": "4500"}]


def test_textfsm_failed_parsing():
    """Verify raw_output is returned if TextFSM template parsing fails."""
    raw_output = "This is not 'show version' output"
    result = utilities.get_structured_data(
        raw_output,
        platform="cisco_ios",
        command="show version",
        template=f"{RESOURCE_FOLDER}/nothinghere",
    )
    assert result == raw_output


def test_textfsm_missing_template():
    """Verify raw_output is returned if TextFSM template is missing."""
    raw_output = "Cisco IOS Software, Catalyst 4500 L3 Switch Software"
    result = utilities.get_structured_data(
        raw_output,
        platform="cisco_ios",
        command="show version",
        template=f"{RESOURCE_FOLDER}/nothinghere",
    )
    assert result == raw_output


def test_get_structured_data_genie():
    """Convert raw CLI output to structured data using Genie"""

    header_line = (
        "Cisco IOS Software, C3560CX Software (C3560CX-UNIVERSALK9-M), "
        "Version 15.2(4)E7, RELEASE SOFTWARE (fc2)"
    )
    raw_output = """
Technical Support: http://www.cisco.com/techsupport
Copyright (c) 1986-2018 by Cisco Systems, Inc.
Compiled Tue 18-Sep-18 13:20 by prod_rel_team

ROM: Bootstrap program is C3560CX boot loader
BOOTLDR: C3560CX Boot Loader (C3560CX-HBOOT-M) Version 15.2(4r)E5, RELEASE SOFTWARE (fc4)

3560CX uptime is 5 weeks, 1 day, 2 hours, 30 minutes
System returned to ROM by power-on
System restarted at 11:45:26 PDT Tue May 7 2019
System image file is "flash:c3560cx-universalk9-mz.152-4.E7.bin"
Last reload reason: power-on



This product contains cryptographic features and is subject to United
States and local country laws governing import, export, transfer and
use. Delivery of Cisco cryptographic products does not imply
third-party authority to import, export, distribute or use encryption.
Importers, exporters, distributors and users are responsible for
compliance with U.S. and local country laws. By using this product you
agree to comply with applicable laws and regulations. If you are unable
to comply with U.S. and local laws, return this product immediately.

A summary of U.S. laws governing Cisco cryptographic products may be found at:
http://www.cisco.com/wwl/export/crypto/tool/stqrg.html

If you require further assistance please contact us by sending email to
export@cisco.com.

License Level: ipservices
License Type: Permanent Right-To-Use
Next reload license Level: ipservices

cisco WS-C3560CX-8PC-S (APM86XXX) processor (revision A0) with 524288K bytes of memory.
Processor board ID FOCXXXXXXXX
Last reset from power-on
5 Virtual Ethernet interfaces
12 Gigabit Ethernet interfaces
The password-recovery mechanism is enabled.

512K bytes of flash-simulated non-volatile configuration memory.
Base ethernet MAC Address       : 12:34:56:78:9A:BC
Motherboard assembly number     : 86-75309-01
Power supply part number        : 867-5309-01
Motherboard serial number       : FOCXXXXXXXX
Power supply serial number      : FOCXXXXXXXX
Model revision number           : A0
Motherboard revision number     : A0
Model number                    : WS-C3560CX-8PC-S
System serial number            : FOCXXXXXXXX
Top Assembly Part Number        : 86-7530-91
Top Assembly Revision Number    : A0
Version ID                      : V01
CLEI Code Number                : CMM1400DRA
Hardware Board Revision Number  : 0x02


Switch Ports Model                     SW Version            SW Image
------ ----- -----                     ----------            ----------
*    1 12    WS-C3560CX-8PC-S          15.2(4)E7             C3560CX-UNIVERSALK9-M


Configuration register is 0xF
"""
    raw_output = header_line + raw_output
    result = utilities.get_structured_data_genie(
        raw_output, platform="cisco_xe", command="show version"
    )
    assert result["version"]["chassis"] == "WS-C3560CX-8PC-S"



#!/usr/bin/env python
from netmiko import ConnectHandler


def main():

    try:
        hostname = raw_input("Enter remote host to test: ")
        username = raw_input("Enter remote username: ")
    except NameError:
        hostname = input("Enter remote host to test: ")
        username = input("Enter remote username: ")

    linux_test = {
        "username": username,
        "use_keys": True,
        "ip": hostname,
        "device_type": "ovs_linux",
        "key_file": "/home/{}/.ssh/test_rsa".format(username),
        "verbose": False,
    }

    net_connect = ConnectHandler(**linux_test)
    print()
    print(net_connect.find_prompt())

    # Test enable mode
    print()
    print("***** Testing enable mode *****")
    net_connect.enable()
    if net_connect.check_enable_mode():
        print("Success: in enable mode")
    else:
        print("Fail...")
    print(net_connect.find_prompt())

    net_connect.exit_enable_mode()
    print("Out of enable mode")
    print(net_connect.find_prompt())

    # Test config mode
    print()
    print("***** Testing config mode *****")
    net_connect.config_mode()
    if net_connect.check_config_mode():
        print("Success: in config mode")
    else:
        print("Fail...")
    print(net_connect.find_prompt())

    net_connect.exit_config_mode()
    print("Out of config mode")
    print(net_connect.find_prompt())

    # Test config mode (when already at root prompt)
    print()
    print("***** Testing config mode when already root *****")
    net_connect.enable()
    if net_connect.check_enable_mode():
        print("Success: in enable mode")
    else:
        print("Fail...")
    print(net_connect.find_prompt())
    print("Test config_mode while already at root prompt")
    net_connect.config_mode()
    if net_connect.check_config_mode():
        print("Success: still at root prompt")
    else:
        print("Fail...")
    net_connect.exit_config_mode()
    # Should do nothing
    net_connect.exit_enable_mode()
    print("Out of config/enable mode")
    print(net_connect.find_prompt())

    # Send config commands
    print()
    print("***** Testing send_config_set *****")
    print(net_connect.find_prompt())
    output = net_connect.send_config_set(["ls -al"])
    print(output)
    print()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""py.test fixtures to be used in netmiko test suite."""
from os import path
import os

import pytest

from netmiko import ConnectHandler, FileTransfer, InLineTransfer, SSHDetect
from tests.test_utils import parse_yaml


PWD = path.dirname(path.realpath(__file__))


def pytest_addoption(parser):
    """Add test_device option to py.test invocations."""
    parser.addoption(
        "--test_device",
        action="store",
        dest="test_device",
        type=str,
        help="Specify the platform type to test on",
    )


@pytest.fixture(scope="module")
def net_connect(request):
    """
    Create the SSH connection to the remote device

    Return the netmiko connection object
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    conn = ConnectHandler(**device)
    return conn


@pytest.fixture(scope="module")
def net_connect_cmd_verify(request):
    """
    Create the SSH connection to the remote device

    Return the netmiko connection object

    Set global_cmd_verify = False
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    device["global_cmd_verify"] = False
    conn = ConnectHandler(**device)
    return conn


@pytest.fixture(scope="function")
def net_connect_newconn(request):
    """
    Create the SSH connection to the remote device

    Return the netmiko connection object.
    Force a new connection for each test.
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    conn = ConnectHandler(**device)
    return conn


@pytest.fixture()
def net_connect_cm(request):
    """
    Create the SSH connection to the remote device using a context manager
    retrieve the find_prompt() data and close the connection.
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    my_prompt = ""
    with ConnectHandler(**device) as conn:
        my_prompt = conn.find_prompt()
    return my_prompt


@pytest.fixture(scope="module")
def net_connect_slog_wr(request):
    """
    Create the SSH connection to the remote device. Modify session_log init arguments.

    Return the netmiko connection object.
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    # Overwrite default session_log location
    device["session_log"] = "SLOG/cisco881_slog_wr.log"
    device["session_log_record_writes"] = True
    conn = ConnectHandler(**device)
    return conn


@pytest.fixture(scope="module")
def device_slog(request):
    """
    Create the SSH connection to the remote device. Modify session_log init arguments.

    Return the netmiko device (not connected)
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    device["session_log_file_mode"] = "append"
    return device


@pytest.fixture(scope="module")
def expected_responses(request):
    """
    Parse the responses.yml file to get a responses dictionary
    """
    device_under_test = request.config.getoption("test_device")
    responses = parse_yaml(PWD + "/etc/responses.yml")
    return responses[device_under_test]


@pytest.fixture(scope="module")
def commands(request):
    """
    Parse the commands.yml file to get a commands dictionary
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    test_platform = device["device_type"]

    commands_yml = parse_yaml(PWD + "/etc/commands.yml")
    return commands_yml[test_platform]


def delete_file_nxos(ssh_conn, dest_file_system, dest_file):
    """
    nxos1# delete bootflash:test2.txt
    Do you want to delete "/test2.txt" ? (yes/no/abort)   [y] y
    """
    if not dest_file_system:
        raise ValueError("Invalid file system specified")
    if not dest_file:
        raise ValueError("Invalid dest file specified")

    full_file_name = "{}{}".format(dest_file_system, dest_file)

    cmd = "delete {}".format(full_file_name)
    output = ssh_conn.send_command_timing(cmd)
    if "yes/no/abort" in output and dest_file in output:
        output += ssh_conn.send_command_timing(
            "y", strip_command=False, strip_prompt=False
        )
        return output
    else:
        output += ssh_conn.send_command_timing("abort")
    raise ValueError("An error happened deleting file on Cisco NX-OS")


def delete_file_ios(ssh_conn, dest_file_system, dest_file):
    """Delete a remote file for a Cisco IOS device."""
    if not dest_file_system:
        raise ValueError("Invalid file system specified")
    if not dest_file:
        raise ValueError("Invalid dest file specified")

    full_file_name = f"{dest_file_system}/{dest_file}"

    cmd = f"delete {full_file_name}"
    output = ssh_conn.send_command_timing(cmd, delay_factor=2)
    if "Delete" in output and dest_file in output:
        output += ssh_conn.send_command_timing("\n", delay_factor=2)
        if "Delete" in output and full_file_name in output and "confirm" in output:
            output += ssh_conn.send_command_timing("y", delay_factor=2)
            return output
        else:
            output += ssh_conn.send_command_timing("n", delay_factor=2)

    raise ValueError("An error happened deleting file on Cisco IOS")


def delete_file_dellos10(ssh_conn, dest_file_system, dest_file):
    """Delete a remote file for a Dell OS10 device."""
    if not dest_file:
        raise ValueError("Invalid dest file specified")

    cmd = "delete home://{}".format(dest_file)
    output = ssh_conn.send_command_timing(cmd)
    if "Proceed to delete" in output:
        output = ssh_conn.send_command_timing("yes")
        return output

    raise ValueError("An error happened deleting file on Dell OS10")


def delete_file_generic(ssh_conn, dest_file_system, dest_file):
    """Delete a remote file for a Junos device."""
    full_file_name = "{}/{}".format(dest_file_system, dest_file)
    cmd = "rm {}".format(full_file_name)
    output = ssh_conn._enter_shell()
    output += ssh_conn.send_command_timing(cmd, strip_command=False, strip_prompt=False)
    output += ssh_conn._return_cli()
    return output


def delete_file_ciena_saos(ssh_conn, dest_file_system, dest_file):
    """Delete a remote file for a ciena device."""
    full_file_name = "{}/{}".format(dest_file_system, dest_file)
    cmd = "file rm {}".format(full_file_name)
    output = ssh_conn.send_command_timing(cmd, strip_command=False, strip_prompt=False)
    return output


@pytest.fixture(scope="module")
def scp_fixture(request):
    """
    Create an FileTransfer object.

    Return a tuple (ssh_conn, scp_handle)
    """
    platform_args = get_platform_args()

    # Create the files
    with open("test9.txt", "w") as f:
        # Not important what it is in the file
        f.write("no logging console\n")

    with open("test2_src.txt", "w") as f:
        # Not important what it is in the file
        f.write("no logging console\n")
        f.write("logging buffered 10000\n")

    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    ssh_conn = ConnectHandler(**device)

    platform = device["device_type"]
    dest_file_system = platform_args[platform]["file_system"]
    if "ciena_saos" in platform and ssh_conn.username:
        dest_file_system = f"/tmp/users/{ssh_conn.username}"

    source_file = "test9.txt"
    dest_file = "test9.txt"
    local_file = "testx.txt"
    direction = "put"

    scp_transfer = FileTransfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=dest_file_system,
        direction=direction,
    )
    scp_transfer.establish_scp_conn()

    # Make sure SCP is enabled
    if platform_args[platform]["enable_scp"]:
        scp_transfer.enable_scp()

    # Delete the test transfer files
    if scp_transfer.check_file_exists():
        func = platform_args[platform]["delete_file"]
        func(ssh_conn, dest_file_system, dest_file)
    if os.path.exists(local_file):
        os.remove(local_file)
    return (ssh_conn, scp_transfer)


@pytest.fixture(scope="module")
def scp_fixture_get(request):
    """
    Create an FileTransfer object (direction=get)

    Return a tuple (ssh_conn, scp_handle)
    """
    platform_args = get_platform_args()
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    ssh_conn = ConnectHandler(**device)

    platform = device["device_type"]
    dest_file_system = platform_args[platform]["file_system"]
    source_file = "test9.txt"
    local_file = "testx.txt"
    dest_file = local_file
    direction = "get"

    scp_transfer = FileTransfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=dest_file_system,
        direction=direction,
    )
    scp_transfer.establish_scp_conn()

    # Make sure SCP is enabled
    if platform_args[platform]["enable_scp"]:
        scp_transfer.enable_scp()

    # Delete the test transfer files
    if os.path.exists(local_file):
        os.remove(local_file)
    return (ssh_conn, scp_transfer)


@pytest.fixture(scope="module")
def tcl_fixture(request):
    """
    Create an InLineTransfer object.

    Return a tuple (ssh_conn, tcl_handle)
    """
    # Create the files
    with open("test9.txt", "w") as f:
        # Not important what it is in the file
        f.write("no logging console\n")

    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    ssh_conn = ConnectHandler(**device)

    dest_file_system = "flash:"
    source_file = "test9.txt"
    dest_file = "test9.txt"
    local_file = "testx.txt"
    direction = "put"

    tcl_transfer = InLineTransfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=dest_file_system,
        direction=direction,
    )

    # Delete the test transfer files
    if tcl_transfer.check_file_exists():
        delete_file_ios(ssh_conn, dest_file_system, dest_file)
    if os.path.exists(local_file):
        os.remove(local_file)

    return (ssh_conn, tcl_transfer)


@pytest.fixture(scope="module")
def ssh_autodetect(request):
    """Create an SSH autodetect object.

    return (ssh_conn, real_device_type)
    """
    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    my_device_type = device.pop("device_type")
    device["device_type"] = "autodetect"
    conn = SSHDetect(**device)
    return (conn, my_device_type)


@pytest.fixture(scope="module")
def scp_file_transfer(request):
    """
    Testing file_transfer

    Return the netmiko connection object
    """
    platform_args = get_platform_args()

    # Create the files
    with open("test9.txt", "w") as f:
        # Not important what it is in the file
        f.write("no logging console\n")

    with open("test2_src.txt", "w") as f:
        # Not important what it is in the file
        f.write("no logging console\n")
        f.write("logging buffered 10000\n")

    device_under_test = request.config.getoption("test_device")
    test_devices = parse_yaml(PWD + "/etc/test_devices.yml")
    device = test_devices[device_under_test]
    device["verbose"] = False
    ssh_conn = ConnectHandler(**device)

    platform = device["device_type"]
    file_system = platform_args[platform]["file_system"]
    source_file = "test9.txt"
    dest_file = "test9.txt"
    local_file = "testx.txt"
    alt_file = "test2.txt"
    direction = "put"

    scp_transfer = FileTransfer(
        ssh_conn,
        source_file=source_file,
        dest_file=dest_file,
        file_system=file_system,
        direction=direction,
    )
    scp_transfer.establish_scp_conn()

    # Delete the test transfer files
    if scp_transfer.check_file_exists():
        func = platform_args[platform]["delete_file"]
        func(ssh_conn, file_system, dest_file)
    if os.path.exists(local_file):
        os.remove(local_file)
    if os.path.exists(alt_file):
        os.remove(alt_file)

    return (ssh_conn, file_system)


def get_platform_args():
    return {
        "cisco_ios": {
            "file_system": "flash:",
            "enable_scp": True,
            "delete_file": delete_file_ios,
        },
        "juniper_junos": {
            "file_system": "/var/tmp",
            "enable_scp": False,
            "delete_file": delete_file_generic,
        },
        "arista_eos": {
            "file_system": "/mnt/flash",
            "enable_scp": False,
            "delete_file": delete_file_generic,
        },
        "cisco_nxos": {
            "file_system": "bootflash:",
            "enable_scp": False,
            "delete_file": delete_file_nxos,
        },
        "cisco_xr": {
            "file_system": "disk0:",
            "enable_scp": False,
            # Delete pattern is the same on IOS-XR
            "delete_file": delete_file_ios,
        },
        "linux": {
            "file_system": "/var/tmp",
            "enable_scp": False,
            "delete_file": delete_file_generic,
        },
        "dellos10": {
            "file_system": "/home/admin",
            "enable_scp": False,
            "delete_file": delete_file_dellos10,
        },
        "ciena_saos": {
            "file_system": "/tmp/users/ciena",
            "enable_scp": False,
            "delete_file": delete_file_ciena_saos,
        },
    }



#!/usr/bin/env python
def test_tcl_put(tcl_fixture):
    ssh_conn, transfer = tcl_fixture
    if transfer.check_file_exists():
        assert False
    else:
        transfer._enter_tcl_mode()
        transfer.put_file()
        transfer._exit_tcl_mode()
        assert transfer.check_file_exists() is True


def test_remote_space_available(tcl_fixture):
    ssh_conn, transfer = tcl_fixture
    remote_space = transfer.remote_space_available()
    assert remote_space >= 30000000


def test_verify_space_available_put(tcl_fixture):
    ssh_conn, transfer = tcl_fixture
    assert transfer.verify_space_available() is True
    # intentional make there not be enough space available
    transfer.file_size = 1000000000
    assert transfer.verify_space_available() is False


def test_remote_file_size(tcl_fixture):
    ssh_conn, transfer = tcl_fixture
    remote_file_size = transfer.remote_file_size()
    assert remote_file_size == 20


def test_md5_methods(tcl_fixture):
    ssh_conn, transfer = tcl_fixture
    md5_value = "4313f1adae86a21117441b0a95d482a7"

    remote_md5 = transfer.remote_md5()
    assert remote_md5 == md5_value
    assert transfer.compare_md5() is True


def test_disconnect(tcl_fixture):
    """Terminate the SSH session."""
    ssh_conn, transfer = tcl_fixture
    ssh_conn.disconnect()



#!/usr/bin/env python
"""
Implement common functions for tests
"""
import io
import sys


def parse_yaml(yaml_file):
    """
    Parses a yaml file, returning its contents as a dict.
    """

    try:
        import yaml
    except ImportError:
        sys.exit("Unable to import yaml module.")

    try:
        with io.open(yaml_file, encoding="utf-8") as fname:
            return yaml.safe_load(fname)
    except IOError:
        sys.exit("Unable to open YAML file: {0}".format(yaml_file))



#!/usr/bin/env python

import time
from os.path import dirname, join
from threading import Lock

from netmiko import NetmikoTimeoutException
from netmiko.base_connection import BaseConnection

RESOURCE_FOLDER = join(dirname(dirname(__file__)), "etc")


class FakeBaseConnection(BaseConnection):
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self._session_locker = Lock()


def test_timeout_exceeded():
    """Raise NetmikoTimeoutException if waiting too much"""
    connection = FakeBaseConnection(session_timeout=10)
    start = time.time() - 11
    try:
        connection._timeout_exceeded(start)
    except NetmikoTimeoutException as exc:
        assert isinstance(exc, NetmikoTimeoutException)
        return

    assert False


def test_timeout_not_exceeded():
    """Do not raise NetmikoTimeoutException if not waiting too much"""
    connection = FakeBaseConnection(session_timeout=10)
    start = time.time()
    assert not connection._timeout_exceeded(start)


def test_timeout_invalid_start():
    """Test invalid timeout start value"""
    connection = FakeBaseConnection(session_timeout=10)
    assert not connection._timeout_exceeded(start=0)


def test_use_ssh_file():
    """Update SSH connection parameters based on the SSH "config" file"""
    connection = FakeBaseConnection(
        host="localhost",
        port=22,
        username="",
        password="secret",
        use_keys=True,
        allow_agent=False,
        key_file="/home/user/.ssh/id_rsa",
        timeout=60,
        pkey=None,
        passphrase=None,
        auth_timeout=None,
        banner_timeout=10,
        ssh_config_file=join(RESOURCE_FOLDER, "ssh_config"),
        sock=None,
    )

    connect_dict = connection._connect_params_dict()

    expected = {
        "hostname": "10.10.10.70",
        "port": 8022,
        "username": "admin",
        "password": "secret",
        "look_for_keys": True,
        "allow_agent": False,
        "key_filename": "/home/user/.ssh/id_rsa",
        "timeout": 60,
        "pkey": None,
        "passphrase": None,
        "auth_timeout": None,
        "banner_timeout": 10,
    }

    result = connection._use_ssh_config(connect_dict)
    assert "sock" in result
    assert len(result["sock"].cmd) == 5
    assert "nc" in result["sock"].cmd
    del result["sock"]
    assert result == expected


def test_use_ssh_file_proxyjump():
    """Update SSH connection parameters based on the SSH "config" file"""
    connection = FakeBaseConnection(
        host="10.10.10.70",
        port=22,
        username="",
        password="secret",
        use_keys=True,
        allow_agent=False,
        key_file="/home/user/.ssh/id_rsa",
        timeout=60,
        pkey=None,
        passphrase=None,
        auth_timeout=None,
        banner_timeout=10,
        ssh_config_file=join(RESOURCE_FOLDER, "ssh_config_proxyjump"),
        sock=None,
    )

    connect_dict = connection._connect_params_dict()

    expected = {
        "hostname": "10.10.10.70",
        "port": 8022,
        "username": "admin",
        "password": "secret",
        "look_for_keys": True,
        "allow_agent": False,
        "key_filename": "/home/user/.ssh/id_rsa",
        "timeout": 60,
        "pkey": None,
        "passphrase": None,
        "auth_timeout": None,
        "banner_timeout": 10,
    }

    result = connection._use_ssh_config(connect_dict)
    assert "sock" in result
    assert "-W" in result["sock"].cmd
    del result["sock"]
    assert result == expected


def test_connect_params_dict():
    """Generate dictionary of Paramiko connection parameters"""
    connection = FakeBaseConnection(
        host="localhost",
        port=22,
        username="user",
        password="secret",
        use_keys=True,
        allow_agent=False,
        key_file="/home/user/.ssh/id_rsa",
        timeout=60,
        pkey=None,
        passphrase=None,
        auth_timeout=None,
        banner_timeout=10,
        ssh_config_file=None,
        sock=None,
    )

    expected = {
        "hostname": "localhost",
        "port": 22,
        "username": "user",
        "password": "secret",
        "look_for_keys": True,
        "allow_agent": False,
        "key_filename": "/home/user/.ssh/id_rsa",
        "timeout": 60,
        "pkey": None,
        "passphrase": None,
        "auth_timeout": None,
        "banner_timeout": 10,
        "sock": None,
    }
    result = connection._connect_params_dict()
    assert result == expected


def test_sanitize_nothing():
    """Keep command echo, trailing router prompt and ANSI escape codes"""

    output = """
show cdp neighbors
Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge
                  S - Switch, H - Host, I - IGMP, r - Repeater, P - Phone,
                  D - Remote, C - CVTA, M - Two-port Mac Relay

Device ID        Local Intrfce     Holdtme    Capability  Platform  Port ID

Total cdp entries displayed : 0
cisco3#"""

    connection = FakeBaseConnection(
        RESPONSE_RETURN="\n",
        RETURN="\n",
        ansi_escape_codes=False,
        base_prompt="cisco3#",
    )

    result = connection._sanitize_output(
        output,
        strip_command=False,
        command_string="show cdp neighbors\n",
        strip_prompt=False,
    )
    assert result == output


def test_sanitize_output():
    """Strip out command echo, trailing router prompt and ANSI escape codes"""

    output = """
show cdp neighbors
Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge
                  S - Switch, H - Host, I - IGMP, r - Repeater, P - Phone,
                  D - Remote, C - CVTA, M - Two-port Mac Relay

Device ID        Local Intrfce     Holdtme    Capability  Platform  Port ID

Total cdp entries displayed : 0
cisco3#"""
    output = output.lstrip()

    expected = """
Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge
                  S - Switch, H - Host, I - IGMP, r - Repeater, P - Phone,
                  D - Remote, C - CVTA, M - Two-port Mac Relay

Device ID        Local Intrfce     Holdtme    Capability  Platform  Port ID

Total cdp entries displayed : 0"""
    expected = expected.lstrip()

    connection = FakeBaseConnection(
        RESPONSE_RETURN="\n",
        RETURN="\n",
        ansi_escape_codes=False,
        base_prompt="cisco3#",
    )

    result = connection._sanitize_output(
        output,
        strip_command=True,
        command_string="show cdp neighbors\n",
        strip_prompt=True,
    )
    assert result == expected


def test_select_global_delay_factor():
    """Select the global delay factor"""
    connection = FakeBaseConnection(global_delay_factor=4, fast_cli=False)
    assert connection.select_delay_factor(2) == 4


def test_select_current_delay_factor():
    """Select the current delay factor"""
    connection = FakeBaseConnection(global_delay_factor=4, fast_cli=False)
    assert connection.select_delay_factor(10) == 10


def test_strip_prompt():
    """Strip the trailing router prompt from the output"""
    string = """MyRouter version 1.25.9
myhostname>"""
    connection = FakeBaseConnection(RESPONSE_RETURN="\n", base_prompt="myhostname>")
    result = connection.strip_prompt(string)
    assert result == "MyRouter version 1.25.9"


def test_strip_no_prompt():
    """Strip no prompt from the output"""
    string = """MyRouter version 1.25.9
additional text"""
    connection = FakeBaseConnection(RESPONSE_RETURN="\n", base_prompt="myhostname>")
    result = connection.strip_prompt(string)
    assert result == string


def test_strip_no_backspaces():
    """Strip no backspace characters out of the output"""
    text = "Writing something"
    output = BaseConnection.strip_backspaces(text)
    assert output == "Writing something"


def test_strip_one_backspaces():
    """Strip one backspace character out of the output"""
    text = "Writing\x08something"
    output = BaseConnection.strip_backspaces(text)
    assert output == "Writingsomething"


def test_strip_backspaces():
    """Strip any backspace characters out of the output"""
    text = "Writing\x08something\x08"
    output = BaseConnection.strip_backspaces(text)
    assert output == "Writingsomething"


def test_strip_command():
    """Strip command string from output"""

    output = """show cdp neighbors
Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge
                  S - Switch, H - Host, I - IGMP, r - Repeater, P - Phone,
                  D - Remote, C - CVTA, M - Two-port Mac Relay

Device ID        Local Intrfce     Holdtme    Capability  Platform  Port ID

Total cdp entries displayed : 0
cisco3#"""

    command = "show cdp neighbors\n"
    expect = """
Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge
                  S - Switch, H - Host, I - IGMP, r - Repeater, P - Phone,
                  D - Remote, C - CVTA, M - Two-port Mac Relay

Device ID        Local Intrfce     Holdtme    Capability  Platform  Port ID

Total cdp entries displayed : 0
cisco3#"""
    expect = expect.lstrip()

    connection = FakeBaseConnection(RESPONSE_RETURN="\n")
    result = connection.strip_command(command, output)
    assert result == expect


def test_strip_command_w_backspaces():
    """Strip command string from output and remove backspaces"""
    output = """echo "hello world"
hello \x08world
myhost\x08name>
"""
    command = 'echo "hello world"'
    expect = """hello world
myhostname>
"""
    connection = FakeBaseConnection(RESPONSE_RETURN="\n")
    result = connection.strip_command(command, output)
    assert result == expect


def test_normalize_linefeeds():
    """Convert combinations of carriage returns and newlines to standardized form."""
    text = """show hostname\r
show version\r\r
show inventory\r\r\r
show interfaces
\r"""
    expected = """show hostname
show version
show inventory
show interfaces
"""
    connection = FakeBaseConnection(RESPONSE_RETURN="\n")
    result = connection.normalize_linefeeds(text)
    assert result == expected


def test_normalize_cmd():
    """Normalize CLI commands to have a single trailing newline"""
    connection = FakeBaseConnection(RETURN="\n")
    result = connection.normalize_cmd("show version \n\n\n ")
    assert result == "show version\n"


def test_unlocking_no_lock():
    """Try unlocking when the lock is already released"""
    connection = FakeBaseConnection()
    assert not connection._session_locker.locked()
    connection._unlock_netmiko_session()
    assert not connection._session_locker.locked()


def test_locking_unlocking():
    """Try to lock and unlock"""
    connection = FakeBaseConnection()
    assert not connection._session_locker.locked()
    connection._lock_netmiko_session()
    assert connection._session_locker.locked()
    connection._unlock_netmiko_session()
    assert not connection._session_locker.locked()


def lock_unlock_timeout(timeout=0):
    """Try to lock when it is already locked"""
    connection = FakeBaseConnection(session_timeout=timeout)
    assert not connection._session_locker.locked()
    connection._lock_netmiko_session()
    assert connection._session_locker.locked()

    try:
        connection._lock_netmiko_session()
    except NetmikoTimeoutException:
        return
    finally:
        assert connection._session_locker.locked()
        connection._unlock_netmiko_session()
        assert not connection._session_locker.locked()

    assert False


def test_lock_timeout():
    """Try to lock when it is already locked"""
    lock_unlock_timeout(0)


def test_lock_timeout_loop():
    """Try to lock when it is already locked, wait 100ms"""
    lock_unlock_timeout(0.2)


def test_strip_ansi_codes():
    connection = FakeBaseConnection(RETURN="\n")
    ansi_codes_to_strip = [
        "\x1b[30m",  # Black
        "\x1b[31m",  # Red
        "\x1b[32m",  # Green
        "\x1b[33m",  # Yellow
        "\x1b[34m",  # Blue
        "\x1b[35m",  # Magenta
        "\x1b[36m",  # Cyan
        "\x1b[37m",  # White
        "\x1b[39m",  # Default(foreground color at startup)
        "\x1b[90m",  # Light Gray
        "\x1b[91m",  # Light Red
        "\x1b[92m",  # Light Green
        "\x1b[93m",  # Light Yellow
        "\x1b[94m",  # Light Blue
        "\x1b[95m",  # Light Magenta
        "\x1b[96m",  # Light Cyan
        "\x1b[97m",  # Light White
        "\x1b[40m",  # Black
        "\x1b[41m",  # Red
        "\x1b[42m",  # Green
        "\x1b[43m",  # Yellow
        "\x1b[44m",  # Blue
        "\x1b[45m",  # Magenta
        "\x1b[46m",  # Cyan
        "\x1b[47m",  # White
        "\x1b[49m",  # Default(background color at startup)
        "\x1b[100m",  # Light Gray
        "\x1b[101m",  # Light Red
        "\x1b[102m",  # Light Green
        "\x1b[103m",  # Light Yellow
        "\x1b[104m",  # Light Blue
        "\x1b[105m",  # Light Magenta
        "\x1b[106m",  # Light Cyan
        "\x1b[107m",  # Light White
        "\x1b[1;5H",  # code_position_cursor r"\[\d+;\d+H"
        "\x1b[?25h",  # code_show_cursor
        "\x1b[K",  # code_erase_line_end
        "\x1b[2K",  # code_erase_line
        "\x1b[K",  # code_erase_start_line
        "\x1b[1;2r",  # code_enable_scroll
        "\x1b[1L",  # code_form_feed
        "\x1b[1M",  # code_carriage_return
        "\x1b[?7l",  # code_disable_line_wrapping
        "\x1b[?7l",  # code_reset_mode_screen_options
        "\x1b[00m",  # code_reset_graphics_mode
        "\x1b[J",  # code_erase_display
        "\x1b[6n",  # code_get_cursor_position
        "\x1b[m",  # code_cursor_position
        "\x1b[J",  # code_erase_display
        "\x1b[0m",  # code_attrs_off
        "\x1b[7m",  # code_reverse
    ]
    for ansi_code in ansi_codes_to_strip:
        assert connection.strip_ansi_escape_codes(ansi_code) == ""

    # code_next_line must be substituted with a return
    assert connection.strip_ansi_escape_codes("\x1bE") == "\n"



#!/usr/bin/env python
def test_ssh_connect(ssh_autodetect):
    """Verify the connection was established successfully."""
    net_conn, real_device_type = ssh_autodetect
    device_type = net_conn.autodetect()
    print(device_type)
    assert device_type == real_device_type



#!/usr/bin/env python
from netmiko import ConnectHandler
from os import path


def main():

    try:
        hostname = raw_input("Enter remote host to test: ")
    except NameError:
        hostname = input("Enter remote host to test: ")

    home_dir = path.expanduser("~")
    key_file = "{}/.ssh/cisco_rsa".format(home_dir)

    cisco_test = {
        "ip": hostname,
        "username": "testuser2",
        "device_type": "cisco_ios",
        "use_keys": True,
        "key_file": key_file,
        "verbose": False,
    }

    net_connect = ConnectHandler(**cisco_test)
    print()
    print("Checking prompt: ")
    print(net_connect.find_prompt())
    print()
    print("Testing show ip int brief: ")
    output = net_connect.send_command("show ip int brief")
    print(output)
    print()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import time
import hashlib
import io
from netmiko import ConnectHandler


def calc_md5(file_name=None, contents=None):
    """Compute MD5 hash of file."""
    if contents is not None:
        pass
    elif file_name:
        with open(file_name, "rb") as f:
            contents = f.read()
    else:
        raise ValueError("Most specify either file_name or contents")

    return hashlib.md5(contents.strip()).hexdigest()


def read_session_log(session_file, append=False):
    """Leading white-space can vary. Strip off leading white-space."""
    with open(session_file, "rb") as f:
        if append is True:
            line = f.readline().decode()
            assert "Initial file contents" in line
        log_content = f.read().lstrip()
        return log_content


def session_action(my_connect, command):
    """Common actions in the netmiko session to generate the session log."""
    time.sleep(1)
    my_connect.clear_buffer()
    output = my_connect.send_command(command)
    my_connect.disconnect()
    return output


def session_log_md5(session_file, compare_file):
    """Compare the session_log MD5 to the compare_file MD5"""
    compare_log_md5 = calc_md5(file_name=compare_file)
    log_content = read_session_log(session_file)
    session_log_md5 = calc_md5(contents=log_content)
    assert session_log_md5 == compare_log_md5


def session_log_md5_append(session_file, compare_file):
    """Compare the session_log MD5 to the compare_file MD5"""
    compare_log_md5 = calc_md5(file_name=compare_file)
    log_content = read_session_log(session_file, append=True)
    session_log_md5 = calc_md5(contents=log_content)
    assert session_log_md5 == compare_log_md5


def test_session_log(net_connect, commands, expected_responses):
    """Verify session_log matches expected content."""
    command = commands["basic"]
    session_action(net_connect, command)

    compare_file = expected_responses["compare_log"]
    session_file = expected_responses["session_log"]

    session_log_md5(session_file, compare_file)


def test_session_log_write(net_connect_slog_wr, commands, expected_responses):
    """Verify session_log matches expected content, but when channel writes are also logged."""
    command = commands["basic"]
    session_action(net_connect_slog_wr, command)

    compare_file = expected_responses["compare_log_wr"]
    session_file = expected_responses["session_log_wr"]
    session_log_md5(session_file, compare_file)


def test_session_log_append(device_slog, commands, expected_responses):
    """Verify session_log matches expected content, but when channel writes are also logged."""
    session_file = expected_responses["session_log_append"]
    # Create a starting file
    with open(session_file, "wb") as f:
        f.write(b"Initial file contents\n\n")

    # The netmiko connection has not been established yet.
    device_slog["session_log"] = session_file

    conn = ConnectHandler(**device_slog)
    command = commands["basic"]
    session_action(conn, command)

    compare_file = expected_responses["compare_log_append"]
    session_log_md5_append(session_file, compare_file)


def test_session_log_bytesio(device_slog, commands, expected_responses):
    """Verify session_log matches expected content, but when channel writes are also logged."""
    s_log = io.BytesIO()

    # The netmiko connection has not been established yet.
    device_slog["session_log"] = s_log
    device_slog["session_log_file_mode"] = "write"

    conn = ConnectHandler(**device_slog)
    command = commands["basic"]
    session_action(conn, command)

    compare_file = expected_responses["compare_log"]
    compare_log_md5 = calc_md5(file_name=compare_file)

    log_content = s_log.getvalue()
    session_log_md5 = calc_md5(contents=log_content)
    assert session_log_md5 == compare_log_md5


def test_session_log_secrets(device_slog):
    """Verify session_log does not contain password or secret."""
    conn = ConnectHandler(**device_slog)
    conn._write_session_log("\nTesting password and secret replacement\n")
    conn._write_session_log("This is my password {}\n".format(conn.password))
    conn._write_session_log("This is my secret {}\n".format(conn.secret))

    if not isinstance(conn.session_log, io.BufferedIOBase):
        with open(conn.session_log.name, "r") as f:
            session_log = f.read()
        if conn.password:
            assert conn.password not in session_log
        if conn.secret:
            assert conn.secret not in session_log
    else:
        assert True



#!/usr/bin/env python


def test_ssh_connect(net_connect, commands, expected_responses):
    """
    Verify the connection was established successfully
    """
    show_version = net_connect.send_command(commands["version"])
    assert expected_responses["version_banner"] in show_version


def test_enable_mode(net_connect, commands, expected_responses):
    """
    Test entering enable mode

    Catch exception for devices that don't support enable
    """
    try:
        net_connect.enable()
        enable_prompt = net_connect.find_prompt()
        assert enable_prompt == expected_responses["enable_prompt"]
    except AttributeError:
        assert True


def test_config_mode(net_connect, commands, expected_responses):
    """
    Test enter config mode
    """
    # Behavior for devices with no config mode is to return null string
    if net_connect.config_mode() != "":
        assert net_connect.check_config_mode() is True
    else:
        assert True


def test_exit_config_mode(net_connect, commands, expected_responses):
    """
    Test exit config mode
    """
    net_connect.exit_config_mode()
    assert net_connect.check_config_mode() is False


def test_config_set(net_connect, commands, expected_responses):
    """Test sending configuration commands."""
    config_commands = commands["config"]
    support_commit = commands.get("support_commit")
    config_verify = commands["config_verification"]

    # Set to initial value and testing sending command as a string
    net_connect.send_config_set(config_commands[0])
    if support_commit:
        net_connect.commit()
    cmd_response = expected_responses.get("cmd_response_init")
    config_commands_output = net_connect.send_command(config_verify)
    if cmd_response:
        assert cmd_response in config_commands_output
    else:
        assert config_commands[0] in config_commands_output
    net_connect.send_config_set(config_commands)
    if support_commit:
        net_connect.commit()
    cmd_response = expected_responses.get("cmd_response_final")
    config_commands_output = net_connect.send_command_expect(config_verify)
    if cmd_response:
        assert cmd_response in config_commands_output
    else:
        assert config_commands[-1] in config_commands_output


def test_config_set_longcommand(net_connect, commands, expected_responses):
    """Test sending configuration commands using long commands"""
    config_commands = commands.get("config_long_command")
    config_verify = commands["config_verification"]  # noqa
    if not config_commands:
        assert True
        return
    output = net_connect.send_config_set(config_commands)  # noqa
    assert True


def test_config_from_file(net_connect, commands, expected_responses):
    """
    Test sending configuration commands from a file
    """
    config_file = commands.get("config_file")
    config_verify = commands["config_verification"]
    if config_file is not None:
        net_connect.send_config_from_file(config_file)
        config_commands_output = net_connect.send_command_expect(config_verify)
        assert expected_responses["file_check_cmd"] in config_commands_output
    else:
        print("Skipping test (no file specified)...")


def test_disconnect(net_connect, commands, expected_responses):
    """
    Terminate the SSH session
    """
    net_connect.disconnect()



#!/usr/bin/env python
import re
import pytest
from ipaddress import ip_address


def test_large_acl(net_connect, acl_entries=100):
    """
    Test creating an ACL with tons of lines
    """

    platforms = {
        "cisco_xe": {
            "base_cmd": "ip access-list extended netmiko_test_large_acl",
            "verify_cmd": "show ip access-lists netmiko_test_large_acl",
            "offset": 4,
        },
        "cisco_ios": {
            "base_cmd": "ip access-list extended netmiko_test_large_acl",
            "verify_cmd": "show ip access-lists netmiko_test_large_acl",
            "offset": 4,
        },
        "cisco_xr": {
            "base_cmd": "ipv4 access-list netmiko_test_large_acl",
            "verify_cmd": "show access-lists netmiko_test_large_acl",
            "offset": 3,
        },
        "cisco_nxos": {
            "base_cmd": "ip access-list netmiko_test_large_acl",
            "verify_cmd": "show ip access-lists netmiko_test_large_acl",
            "offset": 4,
        },
    }

    if net_connect.device_type not in platforms.keys():
        pytest.skip("Platform not supported for ACL test")
    cmd = platforms[net_connect.device_type]["base_cmd"]
    verify_cmd = platforms[net_connect.device_type]["verify_cmd"]
    offset = platforms[net_connect.device_type]["offset"]
    net_connect.send_config_set(f"no {cmd}")
    if "cisco_xr" in net_connect.device_type:
        net_connect.commit()
        net_connect.exit_config_mode()
    cfg_lines = [cmd]

    # Generate sequence of ACL entries
    for i in range(1, acl_entries + 1):
        if "cisco_xr" in net_connect.device_type:
            cmd = f"permit ipv4 host {ip_address('192.168.0.0') + i} any"
        else:
            cmd = f"permit ip host {ip_address('192.168.0.0') + i} any"
        cfg_lines.append(cmd)

    result = net_connect.send_config_set(cfg_lines)
    if "cisco_xr" in net_connect.device_type:
        net_connect.commit()
        net_connect.exit_config_mode()

    # send_config_set should return same num lines + offset lines for entering/exiting cfg-mode
    # NX-OS is will have more than one newline (per line)
    result_list = re.split(r"\n+", result)
    assert len(result_list) == len(cfg_lines) + offset

    # Check that length of lines in show of the acl matches lines configured
    verify = net_connect.send_command(verify_cmd)
    verify_list = re.split(r"\n+", verify.strip())
    # IOS-XR has a timestamp on the show command
    if "UTC" in verify_list[0]:
        verify_list.pop(0)
    assert len(verify_list) == len(cfg_lines)
    net_connect.send_config_set(f"no {cmd}")
    if "cisco_xr" in net_connect.device_type:
        net_connect.commit()
        net_connect.exit_config_mode()
    net_connect.disconnect()



#!/usr/bin/env python
"""
setup_module: setup variables for later use.

test_disable_paging: disable paging
test_ssh_connect: verify ssh connectivity
test_send_command: send a command
test_send_command_timing: send a command
test_base_prompt: test the base prompt
test_strip_prompt: test removing the prompt
test_strip_command: test stripping extraneous info after sending a command
test_normalize_linefeeds: ensure \n is the only line termination character in output
test_clear_buffer: clear text buffer
test_enable_mode: verify enter enable mode
test_disconnect: cleanly disconnect the SSH session
"""
import pytest
import time
from datetime import datetime
from netmiko.utilities import select_cmd_verify


@select_cmd_verify
def bogus_func(obj, *args, **kwargs):
    """Function that just returns the arguments modified by the decorator."""
    return (obj, args, kwargs)


def test_disable_paging(net_connect, commands, expected_responses):
    """Verify paging is disabled by looking for string after when paging would normally occur."""
    if net_connect.device_type == "arista_eos":
        # Arista logging buffer gets enormous
        net_connect.send_command("clear logging")
    multiple_line_output = net_connect.send_command(commands["extended_output"])
    assert expected_responses["multiple_line_output"] in multiple_line_output


def test_ssh_connect(net_connect, commands, expected_responses):
    """Verify the connection was established successfully."""
    show_version = net_connect.send_command(commands["version"])
    assert expected_responses["version_banner"] in show_version


def test_ssh_connect_cm(net_connect_cm, commands, expected_responses):
    """Test the context manager."""
    prompt_str = net_connect_cm
    assert expected_responses["base_prompt"] in prompt_str


def test_send_command_timing(net_connect, commands, expected_responses):
    """Verify a command can be sent down the channel successfully."""
    time.sleep(1)
    net_connect.clear_buffer()
    show_ip = net_connect.send_command_timing(commands["basic"])
    assert expected_responses["interface_ip"] in show_ip
    # Force verification of command echo
    show_ip = net_connect.send_command_timing(commands["basic"], cmd_verify=True)
    assert expected_responses["interface_ip"] in show_ip


def test_send_command(net_connect, commands, expected_responses):
    """Verify a command can be sent down the channel successfully using send_command method."""
    net_connect.clear_buffer()
    show_ip_alt = net_connect.send_command(commands["basic"])
    assert expected_responses["interface_ip"] in show_ip_alt
    show_ip_alt = net_connect.send_command(commands["basic"], cmd_verify=False)
    assert expected_responses["interface_ip"] in show_ip_alt


def test_cmd_verify_decorator(net_connect_cmd_verify):
    obj = net_connect_cmd_verify
    # Global False should have precedence
    assert obj.global_cmd_verify is False
    (obj, args, kwargs) = bogus_func(net_connect_cmd_verify, cmd_verify=True)
    assert kwargs["cmd_verify"] is False
    (obj, args, kwargs) = bogus_func(net_connect_cmd_verify, cmd_verify=False)
    assert kwargs["cmd_verify"] is False

    # Global True should have precedence
    obj.global_cmd_verify = True
    assert obj.global_cmd_verify is True
    (obj, args, kwargs) = bogus_func(net_connect_cmd_verify, cmd_verify=True)
    assert kwargs["cmd_verify"] is True
    (obj, args, kwargs) = bogus_func(net_connect_cmd_verify, cmd_verify=False)
    assert kwargs["cmd_verify"] is True

    # None should track the local argument
    obj.global_cmd_verify = None
    assert obj.global_cmd_verify is None
    (obj, args, kwargs) = bogus_func(net_connect_cmd_verify, cmd_verify=True)
    assert kwargs["cmd_verify"] is True
    (obj, args, kwargs) = bogus_func(net_connect_cmd_verify, cmd_verify=False)
    assert kwargs["cmd_verify"] is False

    # Set it back to proper False value (so later tests aren't messed up).
    obj.global_cmd_verify = False


def test_send_command_global_cmd_verify(
    net_connect_cmd_verify, commands, expected_responses
):
    """
    Verify a command can be sent down the channel successfully using send_command method.

    Disable cmd_verify globally.
    """
    net_connect = net_connect_cmd_verify
    net_connect.clear_buffer()
    # cmd_verify should be disabled globally at this point
    assert net_connect.global_cmd_verify is False
    show_ip_alt = net_connect.send_command(commands["basic"])
    assert expected_responses["interface_ip"] in show_ip_alt


def test_send_command_juniper(net_connect, commands, expected_responses):
    """Verify Juniper complete on space is disabled."""
    # If complete on space is enabled will get re-written to "show ipv6 neighbors"
    if net_connect.device_type == "juniper_junos":
        net_connect.write_channel("show ip neighbors\n")
        output = net_connect.read_until_prompt()
        assert "show ip neighbors" in output
    else:
        assert pytest.skip()


def test_send_command_textfsm(net_connect, commands, expected_responses):
    """Verify a command can be sent down the channel successfully using send_command method."""

    base_platform = net_connect.device_type
    if base_platform.count("_") >= 2:
        # Strip off the _ssh, _telnet, _serial
        base_platform = base_platform.split("_")[:-1]
        base_platform = "_".join(base_platform)
    if base_platform not in [
        "cisco_ios",
        "cisco_xe",
        "cisco_xr",
        "cisco_nxos",
        "arista_eos",
        "cisco_asa",
        "juniper_junos",
        "hp_procurve",
    ]:
        assert pytest.skip("TextFSM/ntc-templates not supported on this platform")
    else:
        time.sleep(1)
        net_connect.clear_buffer()
        fallback_cmd = commands.get("basic")
        command = commands.get("basic_textfsm", fallback_cmd)
        show_ip_alt = net_connect.send_command(command, use_textfsm=True)
        assert isinstance(show_ip_alt, list)


def test_send_command_genie(net_connect, commands, expected_responses):
    """Verify a command can be sent down the channel successfully using send_command method."""

    base_platform = net_connect.device_type
    if base_platform.count("_") >= 2:
        # Strip off the _ssh, _telnet, _serial
        base_platform = base_platform.split("_")[:-1]
        base_platform = "_".join(base_platform)
    if base_platform not in [
        "cisco_ios",
        "cisco_xe",
        "cisco_xr",
        "cisco_nxos",
        "cisco_asa",
    ]:
        assert pytest.skip("Genie not supported on this platform")
    else:
        time.sleep(1)
        net_connect.clear_buffer()
        fallback_cmd = commands.get("basic")
        command = commands.get("basic_textfsm", fallback_cmd)
        show_ip_alt = net_connect.send_command(command, use_genie=True)
        assert isinstance(show_ip_alt, dict)


def test_base_prompt(net_connect, commands, expected_responses):
    """Verify the router prompt is detected correctly."""
    assert net_connect.base_prompt == expected_responses["base_prompt"]


def test_strip_prompt(net_connect, commands, expected_responses):
    """Ensure the router prompt is not in the command output."""

    if expected_responses["base_prompt"] == "":
        return
    show_ip = net_connect.send_command_timing(commands["basic"])
    show_ip_alt = net_connect.send_command(commands["basic"])
    assert expected_responses["base_prompt"] not in show_ip
    assert expected_responses["base_prompt"] not in show_ip_alt


def test_strip_command(net_connect, commands, expected_responses):
    """Ensure that the command that was executed does not show up in the command output."""
    show_ip = net_connect.send_command_timing(commands["basic"])
    show_ip_alt = net_connect.send_command(commands["basic"])

    # dlink_ds has an echo of the command in the command output
    if "dlink_ds" in net_connect.device_type:
        show_ip = "\n".join(show_ip.split("\n")[2:])
        show_ip_alt = "\n".join(show_ip_alt.split("\n")[2:])
    assert commands["basic"] not in show_ip
    assert commands["basic"] not in show_ip_alt


def test_normalize_linefeeds(net_connect, commands, expected_responses):
    """Ensure no '\r\n' sequences."""
    show_version = net_connect.send_command_timing(commands["version"])
    show_version_alt = net_connect.send_command(commands["version"])
    assert "\r\n" not in show_version
    assert "\r\n" not in show_version_alt


def test_clear_buffer(net_connect, commands, expected_responses):
    """Test that clearing the buffer works."""
    # Manually send a command down the channel so that data needs read.
    net_connect.write_channel(commands["basic"] + "\n")
    time.sleep(4)
    net_connect.clear_buffer()

    # Should not be anything there on the second pass
    clear_buffer_check = net_connect.clear_buffer()
    assert clear_buffer_check is None


def test_enable_mode(net_connect, commands, expected_responses):
    """
    Test entering enable mode

    Catch exception for devices that don't support enable
    """
    try:
        net_connect.enable()
        enable_prompt = net_connect.find_prompt()
        assert enable_prompt == expected_responses["enable_prompt"]
    except AttributeError:
        assert True is True


def test_disconnect(net_connect, commands, expected_responses):
    """Terminate the SSH session."""
    start_time = datetime.now()
    net_connect.disconnect()
    end_time = datetime.now()
    time_delta = end_time - start_time
    assert net_connect.remote_conn is None
    assert time_delta.total_seconds() < 8


def test_disconnect_no_enable(net_connect_newconn, commands, expected_responses):
    """Terminate the SSH session from privilege level1"""
    net_connect = net_connect_newconn
    if "cisco_ios" in net_connect.device_type:
        net_connect.send_command_timing("disable")
        start_time = datetime.now()
        net_connect.disconnect()
        end_time = datetime.now()
        time_delta = end_time - start_time
        assert net_connect.remote_conn is None
        assert time_delta.total_seconds() < 5
    else:
        assert True



#!/usr/bin/env python
"""
This will run an command via serial on an cisco ios switch and so
serial cable must be attached to the device
"""
from netmiko import ConnectHandler
import serial


def main():
    """
    This will run an command via serial on an cisco ios switch and so
    serial cable must be attached to the device
    """
    serialhandle = {
        "device_type": "cisco_ios_serial",
        "port": "USB Serial",  # can be COM<number> or any line you can get from
        # serial.tools.list_ports.comports()
        "username": "<username>",
        "password": "<password>",
        "secret": "<secret>",
        "serial_settings": {  # this are the default values
            "baudrate": 9600,
            "bytesize": serial.EIGHTBITS,
            "parity": serial.PARITY_NONE,
            "stopbits": serial.STOPBITS_ONE,
        },
    }
    net_connect = ConnectHandler(**serialhandle)
    net_connect.enable()
    output = net_connect.send_command("show run")
    net_connect.disconnect()

    print(output)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
test_ssh_connect: verify ssh connectivity
test_config_mode: verify enter config mode
test_commit_base: test std .commit()
test_commit_confirm: test commit with confirm
test_confirm_delay: test commit-confirm with non-std delay
test_no_confirm: test commit-confirm with no confirm
test_commit_check: test commit check
test_commit_comment: test commit with comment
test_commit_andquit: test commit andquit
test_exit_config_mode: verify exit config mode
test_disconnect: cleanly disconnect the SSH session
"""

import time
import re
import random
import string


def gen_random(N=6):
    return "".join(
        [
            random.choice(
                string.ascii_lowercase + string.ascii_uppercase + string.digits
            )
            for x in range(N)
        ]
    )


def retrieve_commands(commands):
    """
    Retrieve context needed for a set of commit actions
    """

    config_commands = commands["config"]
    support_commit = commands.get("support_commit")
    config_verify = commands["config_verification"]

    return (config_commands, support_commit, config_verify)


def setup_initial_state(net_connect, commands, expected_responses):
    """
    Setup initial configuration prior to change so that config change can be verified
    """

    # Setup initial state
    config_commands, support_commit, config_verify = retrieve_commands(commands)
    setup_base_config(net_connect, config_commands[0:1])

    cmd_response = expected_responses.get("cmd_response_init", config_commands[0])
    initial_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert initial_state is True

    return config_commands, support_commit, config_verify


def setup_base_config(net_connect, config_changes):
    """
    Send set of config commands and commit
    """
    net_connect.send_config_set(config_changes)
    net_connect.commit()


def config_change_verify(net_connect, verify_cmd, cmd_response):
    """
    Send verify_cmd down channel, verify cmd_response in output
    """
    config_commands_output = net_connect.send_command_expect(verify_cmd)
    if cmd_response in config_commands_output:
        return True
    else:
        return False


def test_ssh_connect(net_connect, commands, expected_responses):
    """
    Verify the connection was established successfully
    """
    show_version = net_connect.send_command_expect(commands["version"])
    assert expected_responses["version_banner"] in show_version


def test_config_mode(net_connect, commands, expected_responses):
    """
    Test enter config mode
    """
    net_connect.config_mode()
    assert net_connect.check_config_mode() is True


def test_commit_base(net_connect, commands, expected_responses):
    """
    Test .commit() with no options
    """
    # Setup the initial config state
    config_commands, support_commit, config_verify = setup_initial_state(
        net_connect, commands, expected_responses
    )

    # Perform commit test
    net_connect.send_config_set(config_commands)
    net_connect.commit()

    cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
    final_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert final_state is True


def test_commit_confirm(net_connect, commands, expected_responses):
    """
    Test confirm with confirm
    """
    # Setup the initial config state
    config_commands, support_commit, config_verify = setup_initial_state(
        net_connect, commands, expected_responses
    )

    # Perform commit-confirm test
    net_connect.send_config_set(config_commands)
    if net_connect.device_type == "cisco_xr":
        net_connect.commit(confirm=True, confirm_delay=60)
    else:
        net_connect.commit(confirm=True)

    cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
    final_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert final_state is True

    # Perform confirm
    net_connect.commit()


def test_confirm_delay(net_connect, commands, expected_responses):
    """
    Test commit with confirm and non-default delay
    """
    # Setup the initial config state
    config_commands, support_commit, config_verify = setup_initial_state(
        net_connect, commands, expected_responses
    )

    # Perform commit-confirm test
    net_connect.send_config_set(config_commands)
    if net_connect.device_type == "cisco_xr":
        net_connect.commit(confirm=True, confirm_delay=60)
    else:
        net_connect.commit(confirm=True, confirm_delay=5)

    cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
    final_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert final_state is True

    # Perform confirm
    net_connect.commit()


def test_no_confirm(net_connect, commands, expected_responses):
    """
    Perform commit-confirm, but don't confirm (verify rollback)
    """
    # Setup the initial config state
    config_commands, support_commit, config_verify = setup_initial_state(
        net_connect, commands, expected_responses
    )

    # Perform commit-confirm test
    net_connect.send_config_set(config_commands)
    if net_connect.device_type == "cisco_xr":
        net_connect.commit(confirm=True, confirm_delay=30)
    else:
        net_connect.commit(confirm=True, confirm_delay=1)

    cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
    final_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert final_state is True

    time.sleep(130)

    # Verify rolled back to initial state
    cmd_response = expected_responses.get("cmd_response_init", config_commands[0])
    init_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert init_state is True


def test_clear_msg(net_connect, commands, expected_responses):
    """
    IOS-XR generates the following message upon a failed commit

    One or more commits have occurred from other
    configuration sessions since this session started
    or since the last commit was made from this session.
    You can use the 'show configuration commit changes'
    command to browse the changes.
    Do you wish to proceed with this commit anyway? [no]: yes

    Clear it
    """
    # Setup the initial config state
    config_commands, support_commit, config_verify = retrieve_commands(commands)

    if net_connect.device_type == "cisco_xr":
        output = net_connect.send_config_set(config_commands)
        output += net_connect.send_command_expect(
            "commit", expect_string=r"Do you wish to"
        )
        output += net_connect.send_command_expect("yes", auto_find_prompt=False)
    assert True is True


def test_commit_check(net_connect, commands, expected_responses):
    """
    Test commit check
    """
    # IOS-XR does not support commit check
    if net_connect.device_type == "cisco_xr":
        assert True is True
    else:

        # Setup the initial config state
        config_commands, support_commit, config_verify = setup_initial_state(
            net_connect, commands, expected_responses
        )

        # Perform commit-confirm test
        net_connect.send_config_set(config_commands)
        net_connect.commit(check=True)

        # Verify at initial state
        cmd_response = expected_responses.get("cmd_response_init", config_commands[0])
        init_state = config_change_verify(net_connect, config_verify, cmd_response)
        assert init_state is True

        rollback = commands.get("rollback")
        net_connect.send_config_set([rollback])


def test_commit_comment(net_connect, commands, expected_responses):
    """
    Test commit with comment
    """
    # Setup the initial config state
    config_commands, support_commit, config_verify = setup_initial_state(
        net_connect, commands, expected_responses
    )

    # Perform commit with comment
    net_connect.send_config_set(config_commands)
    net_connect.commit(comment="Unit test on commit with comment")

    # Verify change was committed
    cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
    final_state = config_change_verify(net_connect, config_verify, cmd_response)
    assert final_state is True

    # Verify commit comment
    commit_verification = commands.get("commit_verification")
    tmp_output = net_connect.send_command(commit_verification)
    if net_connect.device_type == "cisco_xr":
        commit_comment = tmp_output
    else:
        commit_comment = tmp_output.strip().split("\n")[1]
    assert expected_responses.get("commit_comment") in commit_comment.strip()


def test_commit_andquit(net_connect, commands, expected_responses):
    """
    Test commit and immediately quit configure mode
    """

    # IOS-XR does not support commit and quit
    if net_connect.device_type == "cisco_xr":
        assert True is True
    else:

        # Setup the initial config state
        config_commands, support_commit, config_verify = setup_initial_state(
            net_connect, commands, expected_responses
        )

        # Execute change and commit
        net_connect.send_config_set(config_commands)
        net_connect.commit(and_quit=True)

        # Verify change was committed
        cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
        config_verify = "show configuration | match archive"
        final_state = config_change_verify(net_connect, config_verify, cmd_response)
        assert final_state is True


def test_commit_label(net_connect, commands, expected_responses):
    """Test commit label for IOS-XR."""

    if net_connect.device_type != "cisco_xr":
        assert True is True
    else:
        # Setup the initial config state
        config_commands, support_commit, config_verify = setup_initial_state(
            net_connect, commands, expected_responses
        )

        # Execute change and commit
        net_connect.send_config_set(config_commands)
        label = "test_lbl_" + gen_random()
        net_connect.commit(label=label)

        # Verify commit label
        commit_verification = commands.get("commit_verification")
        tmp_output = net_connect.send_command(commit_verification)
        match = re.search(r"Label: (.*)", tmp_output)
        response_label = match.group(1)
        response_label = response_label.strip()
        assert label == response_label


def test_commit_label_comment(net_connect, commands, expected_responses):
    """
    Test commit label for IOS-XR with comment
    """
    # IOS-XR only test
    if net_connect.device_type != "cisco_xr":
        assert True is True
    else:
        # Setup the initial config state
        config_commands, support_commit, config_verify = setup_initial_state(
            net_connect, commands, expected_responses
        )

        # Execute change and commit
        net_connect.send_config_set(config_commands)
        label = "test_lbl_" + gen_random()
        comment = "Test with comment and label"
        net_connect.commit(label=label, comment=comment)

        # Verify commit label
        commit_verification = commands.get("commit_verification")
        tmp_output = net_connect.send_command(commit_verification)
        match = re.search(r"Label: (.*)", tmp_output)
        response_label = match.group(1)
        response_label = response_label.strip()
        assert label == response_label
        match = re.search(r"Comment:  (.*)", tmp_output)
        response_comment = match.group(1)
        response_comment = response_comment.strip()
        response_comment = response_comment.strip('"')
        assert comment == response_comment


def test_commit_label_confirm(net_connect, commands, expected_responses):
    """
    Test commit label for IOS-XR with confirm
    """
    # IOS-XR only test
    if net_connect.device_type != "cisco_xr":
        assert True is True
    else:
        # Setup the initial config state
        config_commands, support_commit, config_verify = setup_initial_state(
            net_connect, commands, expected_responses
        )

        # Execute change and commit
        net_connect.send_config_set(config_commands)
        label = "test_lbl_" + gen_random()
        net_connect.commit(label=label, confirm=True, confirm_delay=120)

        cmd_response = expected_responses.get("cmd_response_final", config_commands[-1])
        final_state = config_change_verify(net_connect, config_verify, cmd_response)
        assert final_state is True

        # Verify commit label
        commit_verification = commands.get("commit_verification")
        tmp_output = net_connect.send_command(commit_verification)
        match = re.search(r"Label: (.*)", tmp_output)
        response_label = match.group(1)
        response_label = response_label.strip()
        assert label == response_label

        net_connect.commit()


def test_exit_config_mode(net_connect, commands, expected_responses):
    """
    Test exit config mode
    """
    net_connect.exit_config_mode()
    time.sleep(1)
    assert net_connect.check_config_mode() is False


def test_disconnect(net_connect, commands, expected_responses):
    """
    Terminate the SSH session
    """
    net_connect.disconnect()



#!/usr/bin/env python
"""
This will run an ssh command successfully on an extreme switch and so SSH must
be enabled on the device
"""
from netmiko import ConnectHandler


def main():
    """
    This will run an ssh command successfully on an extreme switch and so SSH must
    be enabled on the device
    """
    extremehandle = {
        "device_type": "extreme",
        "ip": "10.54.116.175",
        "username": "admin",
        "password": "",
    }
    net_connect = ConnectHandler(**extremehandle)
    output = net_connect.send_command("show config vlan")
    print(output)


if __name__ == "__main__":
    main()



#!/usr/bin/env python


def test_save_base(net_connect, commands, expected_responses):
    """
    Test save config with no options.
    """
    save_verify = expected_responses["save_config"]

    cmd_response = net_connect.save_config()
    assert save_verify in cmd_response


def test_save_confirm(net_connect, commands, expected_responses):
    """
    Test save config with the confirm parameter.
    """
    confirm = commands["save_config_confirm"]
    save_verify = expected_responses["save_config"]

    cmd_response = net_connect.save_config(confirm)
    assert save_verify in cmd_response


def test_save_response(net_connect, commands, expected_responses):
    """
    Test save config with the confirm response parameter.
    """
    confirm_response = commands["save_config_response"]
    save_verify = expected_responses["save_config"]

    cmd_response = net_connect.save_config(confirm_response=confirm_response)
    assert save_verify in cmd_response


def test_save_cmd(net_connect, commands, expected_responses):
    """
    Test save config with cmd parameter.
    """
    cmd = commands["save_config_cmd"]
    save_verify = expected_responses["save_config"]

    cmd_response = net_connect.save_config(cmd=cmd)
    assert save_verify in cmd_response


def test_save_confirm_response(net_connect, commands, expected_responses):
    """
    Test save config with confirm and confirm response parameters
    """
    confirm = commands["save_config_confirm"]
    confirm_response = commands["save_config_response"]
    save_verify = expected_responses["save_config"]

    cmd_response = net_connect.save_config(
        confirm=confirm, confirm_response=confirm_response
    )
    assert save_verify in cmd_response


def test_save_all(net_connect, commands, expected_responses):
    """
    Test the save config method with all additional parameters.
    """
    cmd = commands["save_config_cmd"]
    confirm = commands["save_config_confirm"]
    confirm_response = commands["save_config_response"]
    save_verify = expected_responses["save_config"]

    cmd_response = net_connect.save_config(
        cmd=cmd, confirm=confirm, confirm_response=confirm_response
    )
    assert save_verify in cmd_response


def test_disconnect(net_connect, commands, expected_responses):
    """
    Terminate the SSH session
    """
    net_connect.disconnect()
